package cn.com.qmth.stmms.biz.report.service.impl;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.dao.ExamSubjectDao;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.report.dao.ReportSubjectCollegeDao;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectCollege;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectCollegeService;
import cn.com.qmth.stmms.biz.utils.DoubleUtil;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import java.util.LinkedList;
import java.util.List;

/**
 * 课程学院分析 service实现
 *
 * @author xuwenjin
 * @date 2019-04-16 09:43
 */
@Service
public class ReportSubjectCollegeServiceImpl extends BaseQueryService<ReportSubjectCollege> implements ReportSubjectCollegeService {

    @Autowired
    private ReportSubjectCollegeDao reportSubjectCollegeDao;

    @Autowired
    private ExamSubjectDao examSubjectDao;

    @Transactional
    @Override
    public void deleteByExamId(Integer examId) {
        reportSubjectCollegeDao.deleteByExamId(examId);
    }

    @Override
    public List<ReportSubjectCollege> findByQuery(final ReportSubjectQuery query) {
        List<ReportSubjectCollege> result = reportSubjectCollegeDao.findAll(new Specification<ReportSubjectCollege>() {

            @Override
            public Predicate toPredicate(Root<ReportSubjectCollege> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();
                if (query.getExamId() != null) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (StringUtils.isNotBlank(query.getSubjectCode())) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                if (StringUtils.isNotBlank(query.getCollegeName())) {
                    predicates.add(cb.equal(root.get("collegeName"), query.getCollegeName()));
                }
                return predicates.isEmpty() ? cb.conjunction()
                        : cb.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        });
        return result;
    }

    @Override
    public void saveReportSubjectCollegeData(Integer examId, String subjectCode) {
        ExamSubject examSubject = examSubjectDao.findByExamIdAndCode(examId, subjectCode);
        List<Object[]> reportSubjecCollegetList = reportSubjectCollegeDao.queryReportSubjectCollegeData(examId, subjectCode);
        for (Object[] rs : reportSubjecCollegetList) {
            ReportSubjectCollege subject = new ReportSubjectCollege();
            subject.setExamId(examId);
            subject.setSubjectCode(subjectCode);
            subject.setSubjectName(examSubject.getName());
            subject.setCollegeName(rs[0].toString());
            subject.setMaxScore(Double.parseDouble(rs[1].toString()));
            subject.setMinScore(Double.parseDouble(rs[2].toString()));
            subject.setAvgScore(DoubleUtil.format(Double.parseDouble(rs[3].toString()), "0.00"));
            subject.setPassCount(Integer.parseInt(rs[4].toString()));
            subject.setPassRate(DoubleUtil.div(Double.parseDouble(rs[4].toString()),Integer.parseInt(rs[6].toString()),2));
            subject.setExcellentRate(DoubleUtil.div(Double.parseDouble(rs[5].toString()),Integer.parseInt(rs[6].toString()),2));
            reportSubjectCollegeDao.save(subject);
        }
    }

}
