package cn.com.qmth.stmms.biz.report.service.impl;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.dao.ExamSubjectDao;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.report.dao.ReportSubjectDao;
import cn.com.qmth.stmms.biz.report.dao.ReportSubjectTeacherDao;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectTeacher;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectTeacherService;
import cn.com.qmth.stmms.biz.utils.DoubleUtil;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import java.util.LinkedList;
import java.util.List;

/**
 * 课程学院分析 service实现
 *
 * @author xuwenjin
 * @date 2019-04-16 09:43
 */
@Service
public class ReportSubjectTeacherServiceImpl extends BaseQueryService<ReportSubjectTeacher> implements ReportSubjectTeacherService {

    @Autowired
    private ReportSubjectTeacherDao reportSubjectTeacherDao;

    @Autowired
    private ReportSubjectDao reportSubjectDao;

    @Autowired
    private ExamSubjectDao examSubjectDao;

    @Transactional
    @Override
    public void deleteByExamId(Integer examId) {
        reportSubjectTeacherDao.deleteByExamId(examId);
    }

    @Override
    public List<ReportSubjectTeacher> findByQuery(final ReportSubjectQuery query) {
        List<ReportSubjectTeacher> result = reportSubjectTeacherDao.findAll(new Specification<ReportSubjectTeacher>() {

            @Override
            public Predicate toPredicate(Root<ReportSubjectTeacher> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();

                if (query.getExamId() != null) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (StringUtils.isNotBlank(query.getSubjectCode())) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                if (StringUtils.isNotBlank(query.getTeacherName())) {
                    predicates.add(cb.equal(root.get("teacherName"), query.getTeacherName()));
                }
                return predicates.isEmpty() ? cb.conjunction()
                        : cb.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        });
        return result;
    }

    @Override
    public void saveReportSubjectTeacherData(Integer examId, String subjectCode) {
        ExamSubject examSubject = examSubjectDao.findByExamIdAndCode(examId, subjectCode);
        List<Object[]> reportSubjectTeacherList = reportSubjectTeacherDao.queryReportSubjectTeacherData(examId, subjectCode);
        Double subjectAvgScore = reportSubjectDao.querySubjectAvgScore(examId, subjectCode);
        for (Object[] rs : reportSubjectTeacherList) {
            ReportSubjectTeacher subject = new ReportSubjectTeacher();
            subject.setExamId(examId);
            subject.setSubjectCode(subjectCode);
            subject.setSubjectName(examSubject.getName());
            subject.setTeacherName(rs[0].toString());
            subject.setRealityCount(Integer.parseInt(rs[1].toString()));
            subject.setPassCount(Integer.parseInt(rs[2].toString()));
            subject.setExcellentCount(Integer.parseInt(rs[3].toString()));
            subject.setMaxScore(Double.parseDouble(rs[4].toString()));
            subject.setMinScore(Double.parseDouble(rs[5].toString()));
            subject.setPassRate(DoubleUtil.div(subject.getPassCount(), subject.getRealityCount(), 2));
            subject.setExcellentRate(DoubleUtil.div(subject.getExcellentCount(), subject.getRealityCount(), 2));
            subject.setAvgScore(DoubleUtil.format(Double.parseDouble(rs[6].toString()),"0.00"));
            subject.setRelativeAvgScore(DoubleUtil.format(Double.parseDouble(rs[6].toString()) - subjectAvgScore,"0.00"));
            reportSubjectTeacherDao.save(subject);
        }
    }

}
